@@ -11,7 +11,9 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
+from tests import mock
 from tests import unittest
+from botocore import model
 from botocore.paginate import Paginator
 from botocore.paginate import PaginatorModel
 from botocore.paginate import TokenDecoder
@@ -19,8 +21,6 @@ from botocore.paginate import TokenEncoder
 from botocore.exceptions import PaginationError
 from botocore.compat import six
 
-import mock
-
 
 def encode_token(token):
     return TokenEncoder().encode(token)
@@ -95,12 +95,13 @@ class TestPaginatorModel(unittest.TestCase):
 class TestPagination(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         self.paginate_config = {
             'output_token': 'NextToken',
             'input_token': 'NextToken',
             'result_key': 'Foo',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_result_key_available(self):
         self.assertEqual(
@@ -134,7 +135,7 @@ class TestPagination(unittest.TestCase):
             "result_key": "Users",
             "limit_key": "MaxKeys",
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -173,7 +174,7 @@ class TestPagination(unittest.TestCase):
             'input_token': 'NextToken',
             'result_key': 'Foo',
         }
-        self.paginator = Paginator(self.method, self.pagination_config)
+        self.paginator = Paginator(self.method, self.pagination_config, self.model)
         # Verify that despite varying between NextToken and NextToken2
         # we still can extract the right next tokens.
         responses = [
@@ -203,7 +204,7 @@ class TestPagination(unittest.TestCase):
             'input_token': 'NextToken',
             'result_key': 'Foo',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {'Foo': [1], 'IsTruncated': True, 'NextToken': 'token1'},
             {'Foo': [2], 'IsTruncated': True, 'NextToken': 'token2'},
@@ -225,7 +226,7 @@ class TestPagination(unittest.TestCase):
             'input_token': 'NextToken',
             'result_key': 'Bar',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {'Foo': {'IsTruncated': True}, 'NextToken': 'token1'},
             {'Foo': {'IsTruncated': False}, 'NextToken': 'token2'},
@@ -244,7 +245,7 @@ class TestPagination(unittest.TestCase):
             "result_key": "Users",
             "limit_key": "MaxKeys",
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -285,7 +286,7 @@ class TestPagination(unittest.TestCase):
             "result_key": "Users",
             "limit_key": "MaxKeys",
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -296,17 +297,61 @@ class TestPagination(unittest.TestCase):
         complete = pages.build_full_result()
         self.assertEqual(complete, {'Users': ['User1', 'User2', 'User3']})
 
+    def test_build_multiple_results(self):
+        self.paginate_config = {
+            "output_token": "Marker",
+            "input_token": "Marker",
+            "result_key": "Users",
+            "limit_key": "MaxKeys",
+        }
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
+
+        max_items = 3
+        page_size = 2
+
+        responses = [
+            {"Users": ["User1", "User2"], "Marker": "m1"},
+            {"Users": ["User3", "User4"], "Marker": "m2"},
+            {"Users": ["User3", "User4"], "Marker": "m2"},
+            {"Users": ["User5", "User6", "User7"], "Marker": "m3"},
+        ]
+        self.method.side_effect = responses
+
+        pages = self.paginator.paginate(
+            PaginationConfig={
+                'PageSize': page_size,
+                'MaxItems': max_items
+            }
+        )
+        result = pages.build_full_result()
+
+        pages = self.paginator.paginate(
+            PaginationConfig={
+                'MaxItems': max_items,
+                'PageSize': page_size,
+                'StartingToken': result['NextToken']
+            }
+        )
+        result = pages.build_full_result()
+
+        expected_token = encode_token({
+            'Marker': 'm2',
+            'boto_truncate_amount': 2,
+        })
+        self.assertEqual(expected_token, result['NextToken'])
+
 
 class TestPaginatorPageSize(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         self.paginate_config = {
             "output_token": "Marker",
             "input_token": "Marker",
             "result_key": ["Users", "Groups"],
             'limit_key': 'MaxKeys',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         self.endpoint = mock.Mock()
 
     def test_no_page_size(self):
@@ -331,14 +376,16 @@ class TestPaginatorPageSize(unittest.TestCase):
         kwargs = {'arg1': 'foo', 'arg2': 'bar',
                   'PaginationConfig': {'PageSize': 5}}
         del self.paginate_config['limit_key']
+        paginator = Paginator(self.method, self.paginate_config, self.model)
 
         with self.assertRaises(PaginationError):
-            self.paginator.paginate(**kwargs)
+            paginator.paginate(**kwargs)
 
 
 class TestPaginatorWithPathExpressions(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # This is something we'd see in s3 pagination.
         self.paginate_config = {
             'output_token': [
@@ -346,7 +393,7 @@ class TestPaginatorWithPathExpressions(unittest.TestCase):
             'input_token': 'next_marker',
             'result_key': 'Contents',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_s3_list_objects(self):
         responses = [
@@ -379,12 +426,13 @@ class TestPaginatorWithPathExpressions(unittest.TestCase):
 class TestBinaryTokens(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         self.paginate_config = {
             "output_token": "Marker",
             "input_token": "Marker",
             "result_key": "Users"
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_build_full_result_with_bytes(self):
         responses = [
@@ -549,6 +597,7 @@ class TestBinaryTokens(unittest.TestCase):
 class TestMultipleTokens(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # This is something we'd see in s3 pagination.
         self.paginate_config = {
             "output_token": ["ListBucketResults.NextKeyMarker",
@@ -556,7 +605,7 @@ class TestMultipleTokens(unittest.TestCase):
             "input_token": ["key_marker", "upload_id_marker"],
             "result_key": 'Foo',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_s3_list_multipart_uploads(self):
         responses = [
@@ -590,6 +639,7 @@ class TestOptionalTokens(unittest.TestCase):
     """
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # This is based on Route53 pagination.
         self.paginate_config = {
             "output_token": ["NextRecordName",
@@ -600,7 +650,7 @@ class TestOptionalTokens(unittest.TestCase):
                             "StartRecordIdentifier"],
             "result_key": 'Foo',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_clean_token(self):
         responses = [
@@ -630,13 +680,14 @@ class TestOptionalTokens(unittest.TestCase):
 class TestKeyIterators(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # This is something we'd see in s3 pagination.
         self.paginate_config = {
             "output_token": "Marker",
             "input_token": "Marker",
             "result_key": "Users"
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_result_key_iters(self):
         responses = [
@@ -663,7 +714,7 @@ class TestKeyIterators(unittest.TestCase):
         self.assertEqual(complete, {'Users': ['User1', 'User2', 'User3']})
 
     def test_max_items_can_be_specified(self):
-        paginator = Paginator(self.method, self.paginate_config)
+        paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -679,7 +730,7 @@ class TestKeyIterators(unittest.TestCase):
     def test_max_items_as_strings(self):
         # Some services (route53) model MaxItems as a string type.
         # We need to be able to handle this case.
-        paginator = Paginator(self.method, self.paginate_config)
+        paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -694,7 +745,7 @@ class TestKeyIterators(unittest.TestCase):
             {'Users': ['User1'], 'NextToken': expected_token})
 
     def test_next_token_on_page_boundary(self):
-        paginator = Paginator(self.method, self.paginate_config)
+        paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -711,7 +762,7 @@ class TestKeyIterators(unittest.TestCase):
         # We're saying we only want 4 items, but notice that the second
         # page of results returns users 4-6 so we have to truncated
         # part of that second page.
-        paginator = Paginator(self.method, self.paginate_config)
+        paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1", "User2", "User3"], "Marker": "m1"},
             {"Users": ["User4", "User5", "User6"], "Marker": "m2"},
@@ -731,7 +782,7 @@ class TestKeyIterators(unittest.TestCase):
         # from test_MaxItems_can_be_specified_truncates_response
         # We got the first 4 users, when we pick up we should get
         # User5 - User7.
-        paginator = Paginator(self.method, self.paginate_config)
+        paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User4", "User5", "User6"], "Marker": "m2"},
             {"Users": ["User7"]},
@@ -752,7 +803,7 @@ class TestKeyIterators(unittest.TestCase):
     def test_max_items_exceeds_actual_amount(self):
         # Because MaxItems=10 > number of users (3), we should just return
         # all of the users.
-        paginator = Paginator(self.method, self.paginate_config)
+        paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {"Users": ["User1"], "Marker": "m1"},
             {"Users": ["User2"], "Marker": "m2"},
@@ -771,7 +822,7 @@ class TestKeyIterators(unittest.TestCase):
             {"Users": ["User3"]},
         ]
         self.method.side_effect = responses
-        with self.assertRaisesRegexp(ValueError, 'Bad starting token'):
+        with self.assertRaisesRegex(ValueError, 'Bad starting token'):
             pagination_config = {'StartingToken': 'does___not___work'}
             self.paginator.paginate(
                 PaginationConfig=pagination_config).build_full_result()
@@ -780,13 +831,14 @@ class TestKeyIterators(unittest.TestCase):
 class TestMultipleResultKeys(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # This is something we'd see in s3 pagination.
         self.paginate_config = {
             "output_token": "Marker",
             "input_token": "Marker",
             "result_key": ["Users", "Groups"],
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_build_full_result_with_multiple_result_keys(self):
         responses = [
@@ -901,6 +953,7 @@ class TestMultipleResultKeys(unittest.TestCase):
 class TestMultipleInputKeys(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # Probably the most complicated example we'll see:
         # multiple input/output/result keys.
         self.paginate_config = {
@@ -908,7 +961,7 @@ class TestMultipleInputKeys(unittest.TestCase):
             "input_token": ["InMarker1", "InMarker2"],
             "result_key": ["Users", "Groups"],
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_build_full_result_with_multiple_input_keys(self):
         responses = [
@@ -979,6 +1032,7 @@ class TestMultipleInputKeys(unittest.TestCase):
 class TestExpressionKeyIterators(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         # This is something like what we'd see in RDS.
         self.paginate_config = {
             "input_token": "Marker",
@@ -986,7 +1040,7 @@ class TestExpressionKeyIterators(unittest.TestCase):
             "limit_key": "MaxRecords",
             "result_key": "EngineDefaults.Parameters"
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         self.responses = [
             {"EngineDefaults": {"Parameters": ["One", "Two"]},
              "Marker": "m1"},
@@ -1017,12 +1071,13 @@ class TestExpressionKeyIterators(unittest.TestCase):
 class TestIncludeResultKeys(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         self.paginate_config = {
             'output_token': 'Marker',
             'input_token': 'Marker',
             'result_key': ['ResultKey', 'Count', 'Log'],
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_different_kinds_of_result_key(self):
         self.method.side_effect = [
@@ -1054,13 +1109,14 @@ class TestIncludeNonResultKeys(unittest.TestCase):
 
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         self.paginate_config = {
             'output_token': 'NextToken',
             'input_token': 'NextToken',
             'result_key': 'ResultKey',
             'non_aggregate_keys': ['NotResultKey'],
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
 
     def test_include_non_aggregate_keys(self):
         self.method.side_effect = [
@@ -1079,7 +1135,7 @@ class TestIncludeNonResultKeys(unittest.TestCase):
 
     def test_include_with_multiple_result_keys(self):
         self.paginate_config['result_key'] = ['ResultKey1', 'ResultKey2']
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         self.method.side_effect = [
             {'ResultKey1': ['a', 'b'], 'ResultKey2': ['u', 'v'],
              'NotResultKey': 'a', 'NextToken': 'token1'},
@@ -1102,7 +1158,7 @@ class TestIncludeNonResultKeys(unittest.TestCase):
         self.paginate_config['non_aggregate_keys'] = [
             'Outer', 'Result.Inner',
         ]
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         self.method.side_effect = [
             # The non result keys shows hypothetical
             # example.  This doesn't actually happen,
@@ -1114,7 +1170,7 @@ class TestIncludeNonResultKeys(unittest.TestCase):
             {'Result': {'Key': ['bar', 'baz'], 'Inner': 'v3'},
              'Outer': 'v4', 'NextToken': 't2'},
             {'Result': {'Key': ['qux'], 'Inner': 'v5'},
-             'Outer': 'v6', 'NextToken': 't3'},
+             'Outer': 'v6'},
         ]
         pages = self.paginator.paginate()
         actual = pages.build_full_result()
@@ -1130,13 +1186,14 @@ class TestIncludeNonResultKeys(unittest.TestCase):
 class TestSearchOverResults(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
         self.paginate_config = {
             'more_results': 'IsTruncated',
             'output_token': 'NextToken',
             'input_token': 'NextToken',
             'result_key': 'Foo',
         }
-        self.paginator = Paginator(self.method, self.paginate_config)
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
         responses = [
             {'Foo': [{'a': 1}, {'b': 2}],
              'IsTruncated': True, 'NextToken': '1'},
@@ -1166,6 +1223,7 @@ class TestSearchOverResults(unittest.TestCase):
 class TestDeprecatedStartingToken(unittest.TestCase):
     def setUp(self):
         self.method = mock.Mock()
+        self.model = mock.Mock()
 
     def create_paginator(self, multiple_tokens=False):
         if multiple_tokens:
@@ -1180,7 +1238,7 @@ class TestDeprecatedStartingToken(unittest.TestCase):
                 'input_token': 'Marker',
                 'result_key': 'Users',
             }
-        return Paginator(self.method, paginator_config)
+        return Paginator(self.method, paginator_config, self.model)
 
     def assert_pagination_result(self, expected, pagination_config,
                                  multiple_tokens=False):
@@ -1290,5 +1348,72 @@ class TestDeprecatedStartingToken(unittest.TestCase):
             self.assertEqual(actual, expected)
 
 
+class TestStringPageSize(unittest.TestCase):
+    def setUp(self):
+        self.service_model = {
+            'metadata': {
+                'protocol': 'query',
+                'endpointPrefix': 'prefix'
+            },
+            'documentation': 'best service ever',
+            'operations': {
+                'ListStuff': {
+                    'name': 'ListStuff',
+                    'http': {
+                        'method': 'GET',
+                        'requestUri': '/things'
+                    },
+                    'input': {'shape': 'ListStuffInputShape'},
+                    'output': {'shape': 'ListStuffOutputShape'},
+                    'errors': [],
+                    'documentation': 'Lists stuff'
+                }
+            },
+            'shapes': {
+                'String': {'type': 'string'},
+                'ListOfStuff': {
+                    'type': 'list',
+                    'member': {'type': 'string'}
+                },
+                'ListStuffInputShape': {
+                    'type': 'structure',
+                    'required': [],
+                    'members': {
+                        'NextToken': {'shape': 'String'},
+                        'MaxItems': {'shape': 'String'}
+                    }
+                },
+                'ListStuffOutputShape': {
+                    'type': 'structure',
+                    'required': [],
+                    'members': {
+                        'NextToken': {'shape': 'String'},
+                        'Stuff': {'shape': 'ListOfStuff'},
+                        'IsTruncated': {'type': 'boolean'}
+                    },
+                }
+            }
+        }
+        self.paginate_config = {
+            'input_token': 'NextToken',
+            'output_token': 'NextToken',
+            'limit_key': 'MaxItems',
+            'result_key': 'Stuff',
+        }
+        self.service = model.ServiceModel(self.service_model)
+        self.model = self.service.operation_model('ListStuff')
+        self.method = mock.Mock()
+        self.method.side_effect = [{}]
+        self.paginator = Paginator(self.method, self.paginate_config, self.model)
+
+    def test_int_page_size(self):
+        res = list(self.paginator.paginate(PaginationConfig={'PageSize': 1}))
+        self.method.assert_called_with(MaxItems='1')
+
+    def test_str_page_size(self):
+        res = list(self.paginator.paginate(PaginationConfig={'PageSize': '1'}))
+        self.method.assert_called_with(MaxItems='1')
+
+
 if __name__ == '__main__':
     unittest.main()
