@@ -10,7 +10,6 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-import mock
 import datetime
 import json
 
@@ -31,6 +30,7 @@ from botocore.exceptions import UnsupportedSignatureVersionError
 from botocore.signers import RequestSigner, S3PostPresigner, CloudFrontSigner
 from botocore.signers import generate_db_auth_token
 
+from tests import mock
 from tests import unittest
 from tests import assert_url_equal
 
@@ -434,6 +434,20 @@ class TestSigner(BaseSignerTest):
             region_name='param-override'
         )
 
+    def test_sign_override_signing_name_from_context(self):
+        auth = mock.Mock()
+        auth_types = {
+            'v4': auth
+        }
+        self.request.context = {'signing': {'signing_name': 'override_name'}}
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', self.request)
+        auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='override_name',
+            region_name='region_name'
+        )
+
     def test_sign_with_expires_in(self):
         auth = mock.Mock()
         auth_types = {
