@@ -11,9 +11,7 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import os
-from tests import unittest, BaseEnvVar
-
-import mock
+from tests import mock, unittest, BaseEnvVar
 
 import botocore
 from botocore.compat import six
